@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.702';
+our $VERSION = '1.704';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitSubroutinePrototypes)
@@ -19,10 +19,10 @@ our $VERSION = '1.702';
 
 use Carp ();
 
-my $_has_threads;
+my ($_has_threads, $_tid);
 
 BEGIN {
-   local $@; local $SIG{__DIE__} = \&_NOOP;
+   local $@; local $SIG{__DIE__};
 
    ## Forking is emulated under the Windows enviornment, excluding Cygwin.
    ## MCE 1.514+ will load the 'threads' module by default on Windows.
@@ -37,12 +37,13 @@ BEGIN {
       }
    }
 
-   $_has_threads = $INC{'threads/shared.pm'} ? 1 : 0;
+   $_has_threads = $INC{'threads.pm'} ? 1 : 0;
+   $_tid = $_has_threads ? threads->tid() : 0;
 
    eval 'PDL::no_clone_skip_warning()' if $INC{'PDL.pm'};
 }
 
-use Scalar::Util qw( looks_like_number refaddr );
+use Scalar::Util qw( looks_like_number refaddr weaken );
 use Time::HiRes qw( sleep time );
 
 use Symbol qw( qualify_to_ref );
@@ -75,7 +76,7 @@ BEGIN {
    ## _chunk_id _mce_sid _mce_tid _pids _run_mode _single_dim _thrs _tids _wid
    ## _exiting _exit_pid _total_exited _total_running _total_workers _task_wid
    ## _send_cnt _sess_dir _spawned _state _status _task _task_id _wrk_status
-   ## _init_total_workers _last_sref _mgr_live _rla_data _rla_return
+   ## _init_pid _init_total_workers _last_sref _mgr_live _rla_data _rla_return
    ##
    ## _bsb_r_sock _bsb_w_sock _bse_r_sock _bse_w_sock _com_r_sock _com_w_sock
    ## _dat_r_sock _dat_w_sock _que_r_sock _que_w_sock _rla_r_sock _rla_w_sock
@@ -112,19 +113,19 @@ BEGIN {
       chunk_size max_retries max_workers task_name tmp_dir user_args
    )) {
       *{ $_p } = sub () {
-         my $x = shift; my $self = ref($x) ? $x : $MCE;
+         my $self = shift; $self = $MCE unless ref($self);
          return $self->{$_p};
       };
    }
    for my $_p (qw( chunk_id sess_dir task_id task_wid wid )) {
       *{ $_p } = sub () {
-         my $x = shift; my $self = ref($x) ? $x : $MCE;
+         my $self = shift; $self = $MCE unless ref($self);
          return $self->{"_${_p}"};
       };
    }
    for my $_p (qw( freeze thaw )) {
       *{ $_p } = sub () {
-         my $x = shift; my $self = ref($x) ? $x : $MCE;
+         my $self = shift; $self = $MCE unless ref($self);
          return $self->{$_p}(@_);
       };
    }
@@ -166,8 +167,11 @@ sub import {
 
       if ( $_arg eq 'sereal' ) {
          if (shift eq '1') {
-            local $@; eval 'use Sereal qw(encode_sereal decode_sereal)';
-            $FREEZE = \&encode_sereal, $THAW = \&decode_sereal unless $@;
+            local $@; eval 'use Sereal qw( encode_sereal decode_sereal )';
+            if ( !$@ ) {
+               $FREEZE = sub { encode_sereal( @_, { freeze_callbacks => 1 } ) };
+               $THAW   = \&decode_sereal;
+            }
          }
          next;
       }
@@ -217,7 +221,6 @@ sub import {
 
 use constant {
 
-   FAST_SEND_SIZE => 1024 * 64 + 128,   # Use one print call if N < size
    MAX_CHUNK_SIZE => 1024 * 1024 * 64,  # Maximum chunk size allowed
 
    DATA_CHANNELS  => 8,        # Max data channels
@@ -261,27 +264,20 @@ use constant {
    WANTS_REF      => 3         # Callee wants H/A/S ref
 };
 
-my (%_mce_sess_dir, %_mce_spawned); my $_mce_count = 0;
+my $_mce_count = 0;
 
-MCE::Signal::_set_session_vars(\%_mce_sess_dir, \%_mce_spawned);
-
-sub _clean_sessions {
-   my ($_mce_sid) = @_;
-   for my $_s (keys %_mce_spawned) {
-      delete $_mce_spawned{$_s} unless ($_s eq $_mce_sid);
-   }
-   return;
-}
-sub _clear_session {
-   my ($_mce_sid) = @_;
-   delete $_mce_spawned{$_mce_sid};
-   for my $_s (keys %_mce_spawned) {
-      (delete $_mce_spawned{$_s})->shutdown(1);
-   }
-   return;
+sub CLONE {
+   $_tid = threads->tid() if $_has_threads;
 }
 
-sub DESTROY {}
+sub DESTROY {
+   my $_pid = $_has_threads ? $$ .'.'. $_tid : $$;
+   $_[0]->shutdown(1) if $_[0] && $_[0]->{_init_pid} eq $_pid;
+}
+
+END {
+   $MCE = $TOP_HDLR = undef;
+}
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
@@ -356,7 +352,7 @@ sub new {
       $self{_init_total_workers} = $self{max_workers};
       $self{_chunk_id} = $self{_task_wid} = $self{_wrk_status} = 0;
       $self{_spawned}  = $self{_task_id}  = $self{_wid} = 0;
-      $self{_data_channels} = 1;
+      $self{_init_pid} = 0;
 
       return \%self;
    }
@@ -366,6 +362,8 @@ sub new {
          unless (exists $_valid_fields_new{$_p});
    }
 
+   $self{_init_pid} = ($_has_threads) ? $$ .'.'. $_tid : $$;
+
    if (defined $self{use_threads}) {
       if (!$_has_threads && $self{use_threads} ne '0') {
          my $_msg  = "\n";
@@ -466,15 +464,15 @@ sub new {
 
 sub spawn {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    @_ = ();
 
    _croak('MCE::spawn: method is not allowed by the worker process')
       if ($self->{_wid});
 
-   ## Return if workers have already been spawned.
-   return $self if ($self->{_spawned});
+   ## Return if workers have already been spawned or if module instance.
+   return $self if ($self->{_spawned} || exists $self->{_module_instance});
 
    ## The shared server must be running if present.
    MCE::Shared::start() if ($INC{'MCE/Shared.pm'});
@@ -515,13 +513,11 @@ sub spawn {
       $TOP_HDLR = $self;
    }
    elsif (!$TOP_HDLR->{_mgr_live} && !$TOP_HDLR->{_wid}) {
-      $TOP_HDLR->shutdown if ($_is_MSWin32);
       $TOP_HDLR = $self;
    }
    elsif (refaddr($self) != refaddr($TOP_HDLR)) {
       _croak('Running parallel MCE instances is not supported on Windows')
          if ($_is_MSWin32);
-
       $self->{_data_channels} = 1 if ($self->{_data_channels} > 1);
       $self->{_lock_chn} = 1 if ($self->{_init_total_workers} > 1);
    }
@@ -551,8 +547,6 @@ sub spawn {
 
       $_sess_dir = $self->{_sess_dir} = "$_tmp_dir/$_mce_sid." . (++$_cnt)
          while ( !(mkdir $_sess_dir, 0770) );
-
-      $_mce_sess_dir{$_sess_dir} = 1;
    }
 
    ## -------------------------------------------------------------------------
@@ -593,8 +587,6 @@ sub spawn {
    ## -------------------------------------------------------------------------
 
    ## Spawn workers.
-   $_mce_spawned{$_mce_sid} = $self;
-
    $self->{_pids}   = [], $self->{_thrs}  = [], $self->{_tids} = [];
    $self->{_status} = [], $self->{_state} = [], $self->{_task} = [];
 
@@ -690,7 +682,7 @@ sub forseq {
 
 sub process {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    _validate_runstate($self, 'MCE::process');
 
@@ -738,7 +730,7 @@ sub relay (;&) {
 
 sub restart_worker {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    @_ = ();
 
@@ -783,7 +775,7 @@ sub restart_worker {
 
 sub run {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    _croak('MCE::run: method is not allowed by the worker process')
       if ($self->{_wid});
@@ -994,14 +986,14 @@ sub run {
 
       ## Submit params data to workers.
       for my $_i (1 .. $_total_workers) {
-         print {$_COM_R_SOCK} $_i . $LF;
+         print {$_COM_R_SOCK} $_i.$LF;
          chomp($_wid = <$_COM_R_SOCK>);
 
          if (!$_has_user_tasks || exists $_task0_wids{$_wid}) {
-            print {$_COM_R_SOCK} length($_frozen_params) . $LF . $_frozen_params;
+            print {$_COM_R_SOCK} length($_frozen_params).$LF, $_frozen_params;
             $self->{_state}->[$_wid]->{_params} = \%_params;
          } else {
-            print {$_COM_R_SOCK} length($_frozen_nodata) . $LF . $_frozen_nodata;
+            print {$_COM_R_SOCK} length($_frozen_nodata).$LF, $_frozen_nodata;
             $self->{_state}->[$_wid]->{_params} = \%_params_nodata;
          }
 
@@ -1078,7 +1070,7 @@ sub run {
 
 sub send {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    _croak('MCE::send: method is not allowed by the worker process')
       if ($self->{_wid});
@@ -1123,17 +1115,10 @@ sub send {
    my $_len          = length $_frozen_data;
 
    ## Submit data to worker.
-   print {$_COM_R_SOCK} '_data' . $LF;
-
+   print {$_COM_R_SOCK} '_data'.$LF;
    <$_COM_R_SOCK>;
 
-   if ($_len < FAST_SEND_SIZE) {
-      print {$_COM_R_SOCK} $_len . $LF . $_frozen_data;
-   } else {
-      print {$_COM_R_SOCK} $_len . $LF;
-      print {$_COM_R_SOCK} $_frozen_data;
-   }
-
+   print {$_COM_R_SOCK} $_len.$LF, $_frozen_data;
    <$_COM_R_SOCK>;
 
    if (defined $_submit_delay && $_submit_delay > 0.0) {
@@ -1153,7 +1138,7 @@ sub send {
 
 sub shutdown {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
    my $_no_lock = shift || 0;
 
    @_ = ();
@@ -1162,15 +1147,15 @@ sub shutdown {
    return unless (defined $MCE::Signal::tmp_dir);
    return unless ($self->{_spawned});
 
-   ## Wait for workers to complete processing before shutting down.
    _validate_runstate($self, 'MCE::shutdown');
 
+   ## Wait for workers to complete processing before shutting down.
    $self->run(0) if ($self->{_send_cnt});
 
    local $SIG{__DIE__}  = \&_die;
    local $SIG{__WARN__} = \&_warn;
 
-   lock $_MCE_LOCK if ($_has_threads && ! $_no_lock);
+   lock $_MCE_LOCK if ($_has_threads && !$_no_lock);
 
    my $_COM_R_SOCK     = $self->{_com_r_sock};
    my $_data_channels  = $self->{_data_channels};
@@ -1178,9 +1163,6 @@ sub shutdown {
    my $_sess_dir       = $self->{_sess_dir};
    my $_mce_sid        = $self->{_mce_sid};
 
-   ## Delete entry.
-   delete $_mce_spawned{$_mce_sid};
-
    if (defined $TOP_HDLR && refaddr($self) == refaddr($TOP_HDLR)) {
       $TOP_HDLR = undef;
    }
@@ -1193,7 +1175,7 @@ sub shutdown {
    {
       lock $_EXT_LOCK if $_is_MSWin32;
       for (1 .. $_total_workers) {
-         print {$_COM_R_SOCK} '_exit' . $LF;
+         print {$_COM_R_SOCK} '_exit'.$LF;
          <$_COM_R_SOCK>;
       }
    }
@@ -1237,7 +1219,6 @@ sub shutdown {
          }
       }
       rmdir "$_sess_dir";
-      delete $_mce_sess_dir{$_sess_dir};
    }
 
    ## Reset instance.
@@ -1261,7 +1242,7 @@ sub shutdown {
 
 sub sync {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    return unless ($self->{_wid});
 
@@ -1279,13 +1260,13 @@ sub sync {
    local $\ = undef if (defined $\); local $/ = $LF if (!$/ || $/ ne $LF);
 
    ## Notify the manager process (begin).
-   print {$_DAT_W_SOCK} OUTPUT_B_SYN . $LF . $_chn . $LF;
+   print {$_DAT_W_SOCK} OUTPUT_B_SYN.$LF . $_chn.$LF;
 
    ## Wait here until all workers (task_id 0) have synced.
    1 until sysread $_BSB_R_SOCK, $_buffer, 1;
 
    ## Notify the manager process (end).
-   print {$_DAT_W_SOCK} OUTPUT_E_SYN . $LF . $_chn . $LF;
+   print {$_DAT_W_SOCK} OUTPUT_E_SYN.$LF . $_chn.$LF;
 
    ## Wait here until all workers (task_id 0) have un-synced.
    1 until sysread $_BSE_R_SOCK, $_buffer, 1;
@@ -1295,7 +1276,7 @@ sub sync {
 
 sub yield {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    return unless ($self->{_i_wrk_st});
    return unless ($self->{_task_wid});
@@ -1327,7 +1308,7 @@ sub yield {
 
 sub abort {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    my $_QUE_R_SOCK = $self->{_que_r_sock};
    my $_QUE_W_SOCK = $self->{_que_w_sock};
@@ -1351,11 +1332,11 @@ sub abort {
          $_DAT_LOCK->lock() if $_lock_chn;
 
          if (exists $self->{_rla_return}) {
-            print {$_DAT_W_SOCK} OUTPUT_W_RLA . $LF . $_chn . $LF;
-            print {$_DAU_W_SOCK} (delete $self->{_rla_return}) . $LF;
+            print {$_DAT_W_SOCK} OUTPUT_W_RLA.$LF . $_chn.$LF;
+            print {$_DAU_W_SOCK} (delete $self->{_rla_return}).$LF;
          }
 
-         print {$_DAT_W_SOCK} OUTPUT_W_ABT . $LF . $_chn . $LF;
+         print {$_DAT_W_SOCK} OUTPUT_W_ABT.$LF . $_chn.$LF;
 
          $_DAT_LOCK->unlock() if $_lock_chn;
       }
@@ -1368,7 +1349,7 @@ sub abort {
 
 sub exit {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    my $_exit_status = (defined $_[0]) ? $_[0] : $?;
    my $_exit_msg    = (defined $_[1]) ? $_[1] : '';
@@ -1379,10 +1360,6 @@ sub exit {
    _croak('MCE::exit: method is not allowed by the manager process')
       unless ($self->{_wid});
 
-   MCE::Signal::stop_and_exit('__DIE__') unless ($self->{_running});
-
-   _clear_session( $self->{_mce_sid} );
-
    my $_chn        = $self->{_chn};
    my $_DAT_LOCK   = $self->{_dat_lock};
    my $_DAT_W_SOCK = $self->{_dat_w_sock}->[0];
@@ -1402,31 +1379,34 @@ sub exit {
       $_DAT_LOCK->lock() if $_lock_chn;
 
       if (exists $self->{_rla_return}) {
-         print {$_DAT_W_SOCK} OUTPUT_W_RLA . $LF . $_chn . $LF;
-         print {$_DAU_W_SOCK} (delete $self->{_rla_return}) . $LF;
+         print {$_DAT_W_SOCK} OUTPUT_W_RLA.$LF . $_chn.$LF;
+         print {$_DAU_W_SOCK} (delete $self->{_rla_return}).$LF;
       }
 
-      print {$_DAT_W_SOCK} OUTPUT_W_EXT . $LF . $_chn . $LF;
+      print {$_DAT_W_SOCK} OUTPUT_W_EXT.$LF . $_chn.$LF;
       print {$_DAU_W_SOCK}
-         $_task_id . $LF . $self->{_wid} . $LF . $self->{_exit_pid} . $LF .
-         $_exit_status . $LF . $_exit_id . $LF . $_len . $LF . $_exit_msg
+         $_task_id.$LF . $self->{_wid}.$LF . $self->{_exit_pid}.$LF .
+         $_exit_status.$LF . $_exit_id.$LF . $_len.$LF . $_exit_msg
       ;
 
       if ($self->{_retry} && $self->{_retry}->[2]--) {
          my $_buf = $self->{freeze}($self->{_retry});
-         print {$_DAU_W_SOCK} length($_buf) . $LF . $_buf;
+         print {$_DAU_W_SOCK} length($_buf).$LF, $_buf;
       }
       else {
-         print {$_DAU_W_SOCK} '0' . $LF;
+         print {$_DAU_W_SOCK} '0'.$LF;
       }
 
-      <$_DAU_W_SOCK>;
+      <$_DAU_W_SOCK> unless ($^O eq 'MSWin32');
 
       $_DAT_LOCK->unlock() if $_lock_chn;
    }
 
+   ## Check for any Hobo workers not yet joined.
+   MCE::Hobo->finish() if $INC{'MCE/Hobo.pm'};
+
    ## Exit thread/child process.
-   $SIG{__DIE__} = $SIG{__WARN__} = sub {};
+   $SIG{__DIE__} = $SIG{__WARN__} = sub { };
 
    if ($_has_threads && threads->can('exit')) {
       if ($_is_MSWin32) { lock $_EXT_LOCK; sleep 0.002; }
@@ -1444,7 +1424,7 @@ sub exit {
 
 sub last {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    _croak('MCE::last: method is not allowed by the manager process')
       unless ($self->{_wid});
@@ -1458,7 +1438,7 @@ sub last {
 
 sub next {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    _croak('MCE::next: method is not allowed by the manager process')
       unless ($self->{_wid});
@@ -1472,7 +1452,7 @@ sub next {
 
 sub pid {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    if (defined $self->{_pid}) {
       $self->{_pid};
@@ -1488,7 +1468,7 @@ sub pid {
 
 sub status {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    _croak('MCE::status: method is not allowed by the worker process')
       if ($self->{_wid});
@@ -1506,7 +1486,7 @@ sub status {
 
 sub do {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    _croak('MCE::do: method is not allowed by the manager process')
       unless ($self->{_wid});
@@ -1528,7 +1508,7 @@ sub do {
 
 sub gather {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    _croak('MCE::gather: method is not allowed by the manager process')
       unless ($self->{_wid});
@@ -1551,7 +1531,7 @@ sub gather {
 
    sub sendto {
 
-      my $x = shift; my $self = ref($x) ? $x : $MCE;
+      my $self = shift; $self = $MCE unless ref($self);
       my $_to = shift;
 
       _croak('MCE::sendto: method is not allowed by the manager process')
@@ -1606,8 +1586,8 @@ sub gather {
 
 sub print {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
-   my $_fd = 0; my ($_glob, $_data_ref);
+   my $self = shift; $self = $MCE unless ref($self);
+   my ($_fd, $_glob, $_data_ref);
 
    if (ref $_[0] && defined ($_fd = fileno($_[0]))) {
       $_glob = shift;
@@ -1628,8 +1608,8 @@ sub print {
 
 sub printf {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
-   my $_fd = 0; my ($_glob, $_fmt, $_data);
+   my $self = shift; $self = $MCE unless ref($self);
+   my ($_fd, $_glob, $_fmt, $_data);
 
    if (ref $_[0] && defined ($_fd = fileno($_[0]))) {
       $_glob = shift;
@@ -1645,8 +1625,8 @@ sub printf {
 
 sub say {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
-   my $_fd = 0; my ($_glob, $_data);
+   my $self = shift; $self = $MCE unless ref($self);
+   my ($_fd, $_glob, $_data);
 
    if (ref $_[0] && defined ($_fd = fileno($_[0]))) {
       $_glob = shift;
@@ -1681,7 +1661,7 @@ sub _croak {
 
 sub _get_max_workers {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    if (defined $self->{user_tasks}) {
       if (defined $self->{user_tasks}->[0]->{max_workers}) {
@@ -1698,21 +1678,22 @@ sub _sync_buffer_to_array {
 
    local $_; my $_cnt = 0;
 
-   open my $_MEM_FILE, '<', $_buffer_ref;
-   binmode $_MEM_FILE;
+   open my $_MEM_FH, '<', $_buffer_ref;
+   binmode $_MEM_FH, ':raw';
 
    unless (length $_chop_str) {
-      $_array_ref->[$_cnt++] = $_ while (<$_MEM_FILE>);
+      $_array_ref->[$_cnt++] = $_ while (<$_MEM_FH>);
    }
    else {
-      $_array_ref->[$_cnt++] = <$_MEM_FILE>;
-      while (<$_MEM_FILE>) {
+      $_array_ref->[$_cnt++] = <$_MEM_FH>;
+      while (<$_MEM_FH>) {
          $_array_ref->[$_cnt  ]  = $_chop_str;
          $_array_ref->[$_cnt++] .= $_;
       }
    }
 
-   close $_MEM_FILE; undef $_MEM_FILE;
+   close  $_MEM_FH;
+   weaken $_MEM_FH;
 
    return;
 }
@@ -1770,8 +1751,11 @@ sub _dispatch {
    $self->{_pid} = ($_is_thread) ? $$ .'.'. threads->tid() : $$;
    _worker_main(@_args, \@_plugin_worker_init, $_is_MSWin32);
 
+   ## Check for any Hobo workers not yet joined.
+   MCE::Hobo->finish() if $INC{'MCE/Hobo.pm'};
+
    ## Exit thread/child process.
-   $SIG{__DIE__} = $SIG{__WARN__} = sub {};
+   $SIG{__DIE__} = $SIG{__WARN__} = sub { };
 
    if ($_has_threads && threads->can('exit')) {
       if ($_is_MSWin32) { lock $_EXT_LOCK; sleep 0.002; }
