@@ -11,13 +11,13 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.702';
+our $VERSION = '1.704';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitSubroutinePrototypes)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
-use Scalar::Util qw( looks_like_number );
+use Scalar::Util qw( looks_like_number weaken );
 use Storable ();
 use MCE::Signal;
 
@@ -52,8 +52,11 @@ sub import {
 
       if ( $_arg eq 'sereal' ) {
          if (shift eq '1') {
-            local $@; eval 'use Sereal qw(encode_sereal decode_sereal)';
-            $FREEZE = \&encode_sereal, $THAW = \&decode_sereal unless $@;
+            local $@; eval 'use Sereal qw( encode_sereal decode_sereal )';
+            if ( !$@ ) {
+               $FREEZE = sub { encode_sereal( @_, { freeze_callbacks => 1 } ) };
+               $THAW   = \&decode_sereal;
+            }
          }
          next;
       }
@@ -84,9 +87,7 @@ sub import {
 }
 
 END {
-   return if (defined $_MCE && $_MCE->wid);
-
-   finish();
+   $_MCE = undef;
 }
 
 ###############################################################################
@@ -128,10 +129,12 @@ sub init (@) {
    return;
 }
 
-sub finish () {
+sub finish (@) {
+
+   shift if (defined $_[0] && $_[0] eq 'MCE::Grep');
 
    if (defined $_MCE && $_MCE->{_spawned}) {
-      MCE::_save_state(); $_MCE->shutdown(); MCE::_restore_state();
+      MCE::_save_state(); $_MCE->shutdown(@_); MCE::_restore_state();
    }
 
    $_prev_c = $_total_chunks = undef; undef %_tmp;
@@ -299,6 +302,7 @@ sub run (&@) {
                   binmode $_MEM_FH, ':raw';
                   while (<$_MEM_FH>) { push (@_a, $_) if &{ $_code }; }
                   close   $_MEM_FH;
+                  weaken  $_MEM_FH;
                }
                else {
                   if (ref $_chunk_ref) {
@@ -319,6 +323,7 @@ sub run (&@) {
                   binmode $_MEM_FH, ':raw';
                   while (<$_MEM_FH>) { $_cnt++ if &{ $_code }; }
                   close   $_MEM_FH;
+                  weaken  $_MEM_FH;
                }
                else {
                   if (ref $_chunk_ref) {
@@ -387,7 +392,7 @@ sub run (&@) {
       $MCE::MCE->{_rla_return} = delete $_MCE->{_rla_return};
    }
 
-   if ($^S) {
+   if ($^S || $ENV{'PERL_IPERL_RUNNING'}) {
       ## shutdown if in eval state
       MCE::_save_state(); $_MCE->shutdown(); MCE::_restore_state();
    }
@@ -444,7 +449,7 @@ MCE::Grep - Parallel grep model similar to the native grep function
 
 =head1 VERSION
 
-This document describes MCE::Grep version 1.702
+This document describes MCE::Grep version 1.704
 
 =head1 SYNOPSIS
 
